// NWCreateLoader.cpp : Defines the exported functions for the DLL application.
//

#include "stdafx.h"

#include "nwcreate/LiNwcAll.h"

#include "stdio.h"
#include "stdlib.h"

//Global variables
static LtFloat f_Center_Pt_X = 0;
static LtFloat f_Center_Pt_Y = 0;
static LtFloat f_Center_Pt_Z = 0;
static LtBoolean f_make_cylinder = TRUE;

// Specification of widget read from file
struct WidgetSpec
{
   LtFloat Circle_Radius;
   LtFloat Cylinder_Radius;    
}; 

static LtBoolean parse_file(FILE* fp, WidgetSpec* spec)
{
   if (fscanf(fp, "%lg", &spec->Circle_Radius) != 1)
      return FALSE;
   if (fscanf(fp, "%lg", &spec->Cylinder_Radius) != 1)
      return FALSE;
 
   return TRUE;
}


static LtBoolean LI_NWC_API
geometry_cb(LtNwcGeometry geometry, LtNwcGeometryStream stream_handle,  void* user_data)
{
   LcNwcGeometryStream stream(stream_handle);
   WidgetSpec* spec = static_cast<WidgetSpec*>(user_data);

   // Usefull base vectors
   LtPoint centerPt = { f_Center_Pt_X, f_Center_Pt_Y, f_Center_Pt_Z };
   LtUnitVector Normal = { 0, 0, 1};    
   //draw a circle
   stream.Circle(centerPt,Normal,spec->Circle_Radius);
   //draw a cylinder
   if(f_make_cylinder)
   {
      LtPoint AnotherCenterPt = { f_Center_Pt_X, f_Center_Pt_Y, f_Center_Pt_Z + 500 };
      stream.Cylinder(centerPt,AnotherCenterPt,spec->Cylinder_Radius);
   }  
   stream.End(); 
   return TRUE;
}


static LtBoolean LI_NWC_API 
param_cb(LtNwcLoader loader, LtInt32 which, LtNwcData in_data, void* user_data)
{
   LcNwcData data(in_data);
   switch (which)
   {
   case 0:
      f_Center_Pt_X = data.GetFloat();
      break;
   case 1:
      f_Center_Pt_Y = data.GetFloat();
      break;
   case 2: 
      f_Center_Pt_Z   = data.GetFloat();
      break;
    case 3:
      f_make_cylinder  = data.GetBoolean();
     break;
   default:
      return FALSE;
   }
   return TRUE;
}

static void LI_NWC_API scene_complete_cb(LtNwcScene scene_handle, void* user_data)
{
   WidgetSpec* p = static_cast<WidgetSpec*>(user_data);
   delete p;
} 


static LtNwcLoadStatus LI_NWC_API 
load_file_cb(LtNwcLoader loader, LtWideString pathname, LtNwcScene scene_handle, 
                  LtNwcProgressCallback* progress, void* progress_data, void* user_data)
{
   // Wide string version of fopen is only available on NT so convert to
   // multibyte string and use normal fopen.
   char buffer[1024];
   wcstombs(buffer, pathname, sizeof(buffer));

   //read the loader file to get data
   FILE* fp = fopen(buffer, "r");
   if (!fp)
      return LI_NWC_LOAD_CANT_OPEN;

   WidgetSpec* spec = new WidgetSpec;
   if (!parse_file(fp, spec))
   {
      delete spec;
      return LI_NWC_LOAD_FILE_CORRUPT;
   } 
   //create a scene and geometry
   LcNwcScene scene(scene_handle);
   LcNwcGeometry geom;
 
   //set the callback function for geometry
   geom.SetCallback(&geometry_cb, spec);
   //set the callback function for scene
   scene.SetSceneCompleteCallback(&scene_complete_cb, spec); 
   //add the geometry to the scene
   scene.AddNode(geom); 

   //close the loader file
   fclose(fp);

   return LI_NWC_LOAD_OK;
}


extern "C" __declspec(dllexport) LtBoolean LI_NWC_API 
LiNwcLoaderEntry(LtNwcLoader loader_handle)
{
   LcNwcLoader loader(loader_handle);
   loader.SetLoadFileCallback(&load_file_cb, NULL);
   loader.SetParameterCallback(&param_cb, NULL);   
    return TRUE;  
}
